/*
 * PerlQt interface to qcursor.h
 *
 * Copyright (C) 1997, Ashley Winters <jql@accessone.com>
 *
 * You may distribute under the terms of the LGPL as specified in the
 * README file
 */

#include "pcursor.h"
#include "pbitmap.h"
#include "ppoint.h"
#include "penum.h"

#define STORE_cur(cursor) \
safe_hv_store(hv, MSTR(cursor), \
	      objectify_ptr((void *)&cursor ## Cursor, "QCursor"))

inline void init_const() {
    HV *hv = perl_get_hv("QCursor::Cursor", TRUE | GV_ADDMULTI);

    STORE_cur(arrow);
    STORE_cur(upArrow);
    STORE_cur(cross);
    STORE_cur(wait);
    STORE_cur(ibeam);
    STORE_cur(sizeVer);
    STORE_cur(sizeHor);
    STORE_cur(sizeBDiag);
    STORE_cur(sizeFDiag);
    STORE_cur(sizeAll);
}

#define STORE_sh(key) enumIV(hv, MSTR(key), key ## Cursor)

inline void init_enum() {
    HV *hv = perl_get_hv("QCursor::Shape", TRUE | GV_ADDMULTI);

    STORE_sh(Arrow);
    STORE_sh(UpArrow);
    STORE_sh(Cross);
    STORE_sh(Wait);
    STORE_sh(Ibeam);
    STORE_sh(SizeVer);
    STORE_sh(SizeHor);
    STORE_sh(SizeBDiag);
    STORE_sh(SizeFDiag);
    STORE_sh(SizeAll);
    STORE_sh(Blank);
    STORE_sh(Last);
    STORE_sh(Bitmap);
}


MODULE = QCursor		PACKAGE = QCursor

PROTOTYPES: DISABLE

BOOT:
    init_const();
    init_enum();

PCursor *
PCursor::new(...)
    CASE: items == 1
	CODE:
	RETVAL = new PCursor();
	OUTPUT:
	RETVAL
    CASE: items == 2 && sv_isobject(ST(1))
	PREINIT:
	QCursor *cursor = pextract(QCursor, 1);
	CODE:
	RETVAL = new PCursor(*cursor);
	OUTPUT:
	RETVAL
    CASE: sv_isobject(ST(1))
	PREINIT:
	QBitmap *bitmap = pextract(QBitmap, 1);
	QBitmap *mask = pextract(QBitmap, 2);
	int hotX = (items > 3) ? SvIV(ST(3)) : -1;
	int hotY = (items > 4) ? SvIV(ST(4)) : -1;
	CODE:
	RETVAL = new PCursor(*bitmap, *mask, hotX, hotY);
	OUTPUT:
	RETVAL
    CASE:
	PREINIT:
	int shape = SvIV(ST(1));
	CODE:
	RETVAL = new PCursor(shape);
	OUTPUT:
	RETVAL

void
QCursor::DESTROY()
    CODE:
    if(want_destroy(ST(0)))
	delete THIS;

PBitmap *
QCursor::bitmap()
    CODE:
    RETVAL = new PBitmap(*(THIS->bitmap()));
    OUTPUT:
    RETVAL

PPoint *
QCursor::hotSpot()
    CODE:
    RETVAL = new PPoint(THIS->hotSpot());
    OUTPUT:
    RETVAL

PBitmap *
QCursor::mask()
    CODE:
    RETVAL = new PBitmap(*(THIS->mask()));
    OUTPUT:
    RETVAL

PPoint *
pos()
    CODE:
    RETVAL = new PPoint(QCursor::pos());
    OUTPUT:
    RETVAL

void
setPos(arg1, ...)
    CASE: sv_isobject(ST(0))
	PREINIT:
	QPoint *point = pextract(QPoint, 0);
	CODE:
	QCursor::setPos(*point);
    CASE: items > 1
	PREINIT:
	int x = SvIV(ST(0));
	int y = SvIV(ST(1));
	CODE:
	QCursor::setPos(x, y);

void
QCursor::setShape(shape)
    int shape

int
QCursor::shape()